class ReconnectionPool
  class << self
    include RethinkDB::Shortcuts

    def connect(options)
      @pool ||= Queue.new
      @options = set_default_options(options)

      @options[:pool].times do
        @pool << r.connect(
          host: @options[:host],
          port: @options[:port],
          db: @options[:db],
          auth_key: @options[:auth_key]
        )
      end
    end

    def set_default_options(options)
      options.tap do |o|
        o[:pool] ||= 1
        o[:host] ||= 'localhost'
        o[:port] ||= 28015
        o[:db] ||= 'test'
        o[:auth_key] ||= nil
      end
    end

    def return(conn)
      @pool << conn
    end

    def get
      conn = @pool.pop
      if conn.closed?
        #If the database goes down, you need to be sure that the object knows its state
        conn.close
        begin
          #It keeps the last configuration used
          conn.connect
        rescue Errno::ECONNREFUSED
          # Just loggin', nothing more to do in this case
          p "Can't connect to the database"
        end
      end

      conn
    end

    #Executes a query and returns the connection to the database
    def run(lazy_query)
      conn = self.get
      handle = lazy_query.run(conn)

      self.return conn
      handle
    end

    def empty?
      @pool ||= Queue.new

      @pool.empty?
    end
  end
end
